@@ -1,4 +1,4 @@
 
-from .portfolio import OptimizedWeightingPolicy, OptimizedPortfolio
+from .portfolio import OptimizedPortfolio
 from .portfolio import get_optimized_portfolio
 from .numerics import get_BlackScholesMerton_stocks_estimation
